"""The plugin of the pytest.

The pytest plugin hooks do not need to be imported into any test code, it will
load automatically when running pytest.

References:
    https://docs.pytest.org/en/2.7.3/plugins.html

"""

# pylint: disable=import-error
import os
import sys

import pytest

from rayvision_utils.cmd import Cmd


@pytest.fixture()
def user_info():
    """Get user info."""
    return {
        "domain": "task.renderbus.com",
        "platform": "2",
        "access_id": "df6d1d6s3dc56ds6",
        "access_key": "fa5sd565as2fd65",
        "local_os": 'windows',
        "workspace": "c:/workspace",
        "render_software": "CINEMA 4D",
        "software_version": "R22",
        "project_name": "Project1",
        "plugin_config": {},
    }


@pytest.fixture()
def cg_file_c(tmpdir):
    """Get render cg file."""
    return {
        'cg_file': str(tmpdir.join('ybt.c4d'))
    }


@pytest.fixture()
def handle_cmd():
    """Get a Cmd object."""
    return Cmd()


@pytest.fixture()
def check(task):
    """Create an RayvisionCheck object."""
    from rayvision_api.task.check import RayvisionCheck
    return RayvisionCheck(task)


@pytest.fixture()
def c4d(tmpdir):
    """Create an c4d object."""
    from rayvision_c4d.analyze_c4d import AnalyzeC4d
    if "win" in sys.platform.lower():
        os.environ["USERPROFILE"] = str(tmpdir)
    else:
        os.environ["HOME"] = str(tmpdir)
    return AnalyzeC4d(str(tmpdir), "R22", "Project1", {})



# -*- coding: utf-8 -*-
"""C4d render demo."""

from rayvision_api.core import RayvisionAPI
from rayvision_api.task.check import RayvisionCheck
from rayvision_api.utils import update_task_info, append_to_task, append_to_upload
from rayvision_c4d.analyze_c4d import AnalyzeC4d
from rayvision_sync.download import RayvisionDownload
from rayvision_sync.upload import RayvisionUpload

# API Parameter
render_para = {
    "domain": "task.renderbus.com",
    "platform": "6",
    "access_id": "xxxxx",
    "access_key": "xxxxxx",
}

api = RayvisionAPI(access_id=render_para['access_id'],
                   access_key=render_para['access_key'],
                   domain=render_para['domain'],
                   platform=render_para['platform'])

# Step1:Analyze CG File
analyze_info = {
    "cg_file": r"D:\houdini\cg_file\ybt.c4d",
    "workspace": "c:/workspace",
    "software_version": "R22",
    "project_name": "Project1",
    "plugin_config": {},
    "platform": render_para['platform']
}
analyze_obj = AnalyzeC4d(**analyze_info)
analyze_obj.analyse(exe_path=r"C:\Program Files\Maxon Cinema 4D R22\Cinema 4D.exe")

# Step2:Add some custom parameters, or update the original parameter value
update_task = {
    "pre_frames": "100",
    "stop_after_test": "1"
}
update_task_info(update_task, analyze_obj.task_json)

custom_info_to_task = {}
append_to_task(custom_info_to_task, analyze_obj.task_json)

custom_info_to_upload = []
append_to_upload(custom_info_to_upload, analyze_obj.upload_json)

# Step3: Set platform hardware configuration information
hardware_config = {
    "model": "Default",  # Platform CPU: Default or Platform GPU: 1080Ti or 2080Ti
    "ram": "128GB",  # memory: 64GB or 128GB
    "gpuNum": None  # GPU platform requires input like 2*GPU, if CPU platform it is None
}

# Step4:Check json files
check_obj = RayvisionCheck(api, analyze_obj)
task_id = check_obj.execute(hardware_config, analyze_obj.task_json, analyze_obj.upload_json)

# Step5:Transmission
"""
There are two ways to upload the transmission:
Upload_method: 1:upload four json files and upload the resource file according to upload.json;
               2:json files and resources are uploaded separately;
"""
CONFIG_PATH = {
    "tips_json_path": analyze_obj.tips_json,
    "task_json_path": analyze_obj.task_json,
    "asset_json_path": analyze_obj.asset_json,
    "upload_json_path": analyze_obj.upload_json,
}
upload_obj = RayvisionUpload(api, automatic_line=True)
"""
The default of the test demo is to upload json and resource files at the same time,
and users can choose their own upload method according to the actual situation.
"""
upload_method = 1
if upload_method == 1:
    # step5.1:Json files are uploaded in conjunction with CG resources
    upload_obj.upload(str(task_id), **CONFIG_PATH)
elif upload_method == 2:
    # step5.2:CG resource files and json are uploaded separately
    upload_obj.upload_asset(upload_json_path=CONFIG_PATH["upload_json_path"])
    upload_obj.upload_config(str(task_id), list(CONFIG_PATH.values()))

# Step6:Submit Task
api.submit(int(task_id))

# Step7:Download
download = RayvisionDownload(api)
# All complete before the automatic start of uniform download.
# download.auto_download_after_task_completed([task_id])
# Poll download (automatic download for each completed frame)
download.auto_download([int(task_id)])



# -*- coding: utf-8 -*-
"""A interface for c4d."""

# Import built-in models
from __future__ import print_function
from __future__ import unicode_literals

import base64
import hashlib
import logging
import os
import sys
import time
from builtins import str
import threading

from rayvision_c4d.constants import PACKAGE_NAME
from rayvision_c4d.get_preferences import GetInstallPath
from rayvision_log import init_logger
from rayvision_utils import constants
from rayvision_utils import utils
from rayvision_utils.cmd import Cmd
from rayvision_utils.exception import tips_code
from rayvision_utils.exception.exception import AnalyseFailError, CGFileNotExistsError

VERSION = sys.version_info[0]


class AnalyzeC4d(object):
    def __init__(self, cg_file, software_version, project_name,
                 plugin_config, render_software="CINEMA 4D", render_layer_type="0",
                 input_project_path=None, local_os=None, workspace=None,
                 custom_exe_path=None,
                 platform="2",
                 logger=None,
                 log_folder=None,
                 log_name=None,
                 log_level="DEBUG"
                 ):
        """Initialize and examine the analysis information.

        Args:
            cg_file (str): Scene file path.
            software_version (str): Software version.
            project_name (str): The project name.
            plugin_config (dict): Plugin information.
            render_software (str): Software name, CINEMA 4D by default.
            render_layer_type (str): 0 is render layer, 1 is render setup.
            input_project_path (str): The working path of the scenario.
            local_os (str): System name, linux or windows.
            workspace (str): Analysis out of the result file storage path.
            custom_exe_path (str): Customize the exe path for the analysis.
            platform (str): Platform num.
            logger (object, optional): Custom log object.
            log_folder (str, optional): Custom log save location.
            log_name (str, optional): Custom log file name.
            log_level (string):  Set log level, example: "DEBUG","INFO","WARNING","ERROR".
        """
        self.logger = logger
        if not self.logger:
            init_logger(PACKAGE_NAME, log_folder, log_name)
            self.logger = logging.getLogger(__name__)
            self.logger.setLevel(level=log_level.upper())

        self.check_path(cg_file)
        self.cg_file = cg_file

        self.render_software = render_software
        self.input_project_path = input_project_path or ""
        self.render_layer_type = render_layer_type
        self.software_version = software_version
        self.project_name = project_name
        self.plugin_config = plugin_config

        local_os = self.check_local_os(local_os)
        self.local_os = local_os
        self.tmp_mark = str(int(time.time())) + str(self.get_current_id())
        workspace = os.path.join(self.check_workspace(workspace),
                                 self.tmp_mark)
        if not os.path.exists(workspace):
            os.makedirs(workspace)
        self.workspace = workspace

        if custom_exe_path:
            self.check_path(custom_exe_path)
        self.custom_exe_path = custom_exe_path

        self.platform = platform

        self.task_json = os.path.join(workspace, "task.json")
        self.tips_json = os.path.join(workspace, "tips.json")
        self.asset_json = os.path.join(workspace, "asset.json")
        self.upload_json = os.path.join(workspace, "upload.json")
        self.analyse_log_path = workspace
        self.tips_info = {}
        self.task_info = {}
        self.asset_info = {}
        self.upload_info = {}

    @staticmethod
    def get_current_id():
        if isinstance(threading.current_thread(), threading._MainThread):
            return os.getpid()
        else:
            return threading.get_ident()

    @staticmethod
    def check_path(tmp_path):
        """Check if the path exists."""
        if not os.path.exists(tmp_path):
            raise CGFileNotExistsError("{} is not found".format(tmp_path))

    def add_tip(self, code, info):
        """Add error message.
        
        Args:
            code (str): error code.
            info (str or list): Error message description.

        """
        if isinstance(info, str):
            self.tips_info[code] = [info]
        elif isinstance(info, list):
            self.tips_info[code] = info
        else:
            raise Exception("info must a list or str.")

    def save_tips(self):
        """Write the error message to tips.json."""
        utils.json_save(self.tips_json, self.tips_info, ensure_ascii=False)

    @staticmethod
    def check_local_os(local_os):
        """Check the system name.

        Args:
            local_os (str): System name.

        Returns:
            str

        """
        if not local_os:
            if "win" in sys.platform.lower():
                local_os = "windows"
            else:
                local_os = "linux"
        return local_os

    def check_workspace(self, workspace):
        """Check the working environment.

        Args:
            workspace (str):  Workspace path.

        Returns:
            str: Workspace path.

        """
        if not workspace:
            if self.local_os == "windows":
                workspace = os.path.join(os.environ["USERPROFILE"], "renderfarm_sdk")
            else:
                workspace = os.path.join(os.environ["HOME"], "renderfarm_sdk")
        else:
            self.check_path(workspace)

        return workspace

    def analyse_cg_file(self):
        """Analyse cg file.

        Analyze the scene file to get the path to the startup file of the CG
        software.

        """
        # Find the version from the cg file
        if VERSION == 3:
            version = self.check_version3(self.cg_file)
        else:
            version = self.check_version2(self.cg_file)

        if int(float(version)) != int(float(self.software_version)):
            self.add_tip(tips_code.CG_NOTMATCH, "{0} {1}".format(
                self.render_software, self.software_version))
            self.save_tips()

        # Find the installation path with the version
        if self.custom_exe_path is not None:
            exe_path = self.custom_exe_path
        else:
            exe_path = self.find_location()

        return exe_path

    def write_task_json(self):
        """The initialization task.json."""
        constants.TASK_INFO["task_info"]["input_cg_file"] = self.cg_file.replace("\\", "/")
        constants.TASK_INFO["task_info"]["input_project_path"] = self.input_project_path.replace("\\", "/")
        constants.TASK_INFO["task_info"]["render_layer_type"] = self.render_layer_type
        constants.TASK_INFO["task_info"]["project_name"] = self.project_name
        constants.TASK_INFO["task_info"]["cg_id"] = "2005"
        constants.TASK_INFO["task_info"]["os_name"] = "1" if self.local_os == "windows" else "0"
        constants.TASK_INFO["task_info"]["platform"] = self.platform
        constants.TASK_INFO["software_config"] = {
            "plugins": self.plugin_config,
            "cg_version": self.software_version,
            "cg_name": self.render_software
        }
        utils.json_save(self.task_json, constants.TASK_INFO)

    def check_result(self):
        """Check that the analysis results file exists."""
        for json_path in [self.task_json, self.asset_json,
                          self.tips_json]:
            if not os.path.exists(json_path):
                msg = "Json file is not generated: {0}".format(json_path)
                return False, msg
        return True, None

    def get_file_md5(self, file_path):
        """Generate the md5 values for the scenario."""
        hash_md5 = hashlib.md5()
        if os.path.exists(file_path):
            with open(file_path, 'rb') as file_path_f:
                while True:
                    data_flow = file_path_f.read(8096)
                    if not data_flow:
                        break
                    hash_md5.update(data_flow)
        return hash_md5.hexdigest()

    def write_upload_json(self):
        """Generate the upload.json."""
        assets = self.asset_info["asset"]
        upload_asset = []

        self.upload_info["scene"] = [
            {
                "local": self.cg_file.replace("\\", "/"),
                "server": utils.convert_path(self.cg_file),
                "hash": self.get_file_md5(self.cg_file)
            }
        ]

        for path in assets:
            resources = {}
            local = path.split("  (mtime")[0]
            server = utils.convert_path(local)
            resources["local"] = local.replace("\\", "/")
            resources["server"] = server
            upload_asset.append(resources)

        # Add the cg file to upload.json
        upload_asset.append({
            "local": self.cg_file.replace("\\", "/"),
            "server": utils.convert_path(self.cg_file)
        })

        self.upload_info["asset"] = upload_asset

        utils.json_save(self.upload_json, self.upload_info)

    def __copy_file(self, src, dst):
        copy_cmd = 'xcopy /s /y /f /e "%s" "%s"' % (src, dst)
        print('Copy command: [%s]' % copy_cmd)
        os.system(copy_cmd)

    def update_pyp_script(self, exe_path, cg_ver):
        print('Update analyze pyp...')
        curr_dir = os.path.dirname(__file__)
        base_dir = os.path.abspath(curr_dir)

        src_plugin = os.path.join(base_dir, 'tool')

        maxon_temp_path = os.path.join(os.getenv('APPDATA'), 'MAXON')

        if not os.path.exists(maxon_temp_path):
            os.makedirs(maxon_temp_path)

        flag = False
        for dir in os.listdir(maxon_temp_path):
            lower_dir = dir.lower()
            lower_inst = os.path.basename(os.path.dirname(exe_path)).lower()
            lower_ver = cg_ver.lower()
            print(lower_dir, lower_inst, lower_ver)
            if lower_dir.startswith(lower_ver) or lower_dir.startswith(lower_inst):
                flag = True
                maxon_plugin_path = os.path.join(maxon_temp_path, dir, 'plugins')

                if not os.path.exists(maxon_plugin_path):
                    os.makedirs(maxon_plugin_path)
                else:
                    try:
                        os.remove(os.path.join(maxon_plugin_path, 'RBAnalyzer.pyp'))
                        os.system('del /q /s %s\\python26\\*' % maxon_plugin_path)
                        os.system('del /q /s %s\\python27\\*' % maxon_plugin_path)
                        os.system('del /q /s %s\\python37\\*' % maxon_plugin_path)
                    except:
                        pass

                print('Copy pyp: from [%s] to [%s]' % (src_plugin, maxon_plugin_path))

                try:
                    self.__copy_file(src_plugin, maxon_plugin_path)
                    print('RBAnalyzer.pyp was updated...')
                except:
                    pass

        if not flag:
            path_finder = GetInstallPath()
            pref_path_inst = path_finder.install_path(os.path.dirname(exe_path))
            if not os.path.exists(pref_path_inst):
                os.makedirs(pref_path_inst)

            flag = self.update_pyp_script(exe_path, cg_ver)

        return flag

    def analyse(self, exe_path):
        """Build a cmd command to perform an analysis scenario.

        Args:
            exe_path (bool): Do you not generate an upload,json file.

        Raises:
            AnalyseFailError: Analysis scenario failed.

        """
        if not os.path.exists(exe_path):
            self.logger.error("Please enter the c4d software absolute path")
            raise AnalyseFailError

        cg_ver = '{} {}'.format(self.render_software, self.software_version)  # Cinema 4D R19
        if not self.update_pyp_script(exe_path, cg_ver):
            print('[ERROR] MAXON appdata "%appdata%/MAXON" not found')
            raise ValueError('MAXON appdata not found')

        self.write_task_json()

        print('Analyze cg file: [%s]' % self.cg_file)
        # if sys.version_info.major == 2:
        #     cg_file = base64.b64encode(bytes(self.cg_file)).decode("utf-8")
        # else:
        #     cg_file = base64.b64encode(bytes(self.cg_file, 'utf-8')).decode("utf-8")
        # print('Encoded cg file: [%s]' % cg_file)

        if self.local_os == 'windows':
            cmd = ('"{exe_path}" -cg_file="{cg_file}" -task_json="{task_json}" '
                   '-asset_json="{asset_json}" -tips_json="{tips_json}" -upload_json="{upload_json}" '
                   '-log_path="{log_path}" '
                   '-parallel -nogui').format(
                exe_path=exe_path,
                cg_file=self.cg_file,
                task_json=self.task_json,
                asset_json=self.asset_json,
                tips_json=self.tips_json,
                upload_json=self.upload_json,
                log_path=self.analyse_log_path
            )

        else:
            self.logger.error("c4d does not support linux rendering")

        self.logger.debug(cmd)
        code, _, _ = Cmd.run(cmd, shell=True)
        if code not in [0, 1]:
            self.add_tip(tips_code.UNKNOW_ERR, "")
            self.save_tips()
            raise AnalyseFailError

        # Determine whether the analysis is successful by
        #  determining whether a json file is generated.
        status, msg = self.check_result()
        if status is False:
            self.add_tip(tips_code.UNKNOW_ERR, msg)
            self.save_tips()
            raise AnalyseFailError(msg)

        self.tips_info = utils.json_load(self.tips_json)
        self.asset_info = utils.json_load(self.asset_json)
        self.task_info = utils.json_load(self.task_json)



# -*- coding:utf-8 -*-
"""Test rayvision_c4d.utils functions."""

# pylint: disable=import-error
import pytest

from rayvision_c4d import utils


@pytest.mark.parametrize("message, py_version", [
    ("test to encode", 3),
])
def test_get_encode(message, py_version):
    """Test get_encode, we can get a expected result."""
    assert utils.get_encode(message, py_version) == "unicode"


@pytest.mark.parametrize("message, py_version", [
    ("test str to unicode", 3),
])
def test_str_to_unicode(message, py_version):
    """Test str_to_unicode, we can get a expected type."""
    result = utils.str_to_unicode(message, py_version)
    assert isinstance(result, str)


@pytest.mark.parametrize("str1, py_version", [
    ("test unicode to str", 3),
])
def test_unicode_to_str(str1, py_version):
    """Test str_to_unicode, we can get a expected type."""
    result = utils.unicode_to_str(str1, py_version=py_version)
    assert isinstance(result, str)


@pytest.mark.parametrize("str1, py_version, str_decode", [
    (b"test unicode to str", 3, 'utf-8'),
])
def test_bytes_to_str(str1, py_version, str_decode):
    """Test str_to_unicode, we can get a expected type."""
    result = utils.bytes_to_str(str1, py_version=py_version,
                                str_decode=str_decode)
    assert isinstance(result, str)


@pytest.mark.parametrize("encode_str, py_version", [
    ("to gbk", 3),
])
def test_to_gbk(encode_str, py_version):
    """Test str_to_unicode, we can get a expected type."""
    result = utils.to_gbk(encode_str, py_version)
    assert isinstance(result, str)


@pytest.mark.parametrize("path", [
    "D:/work/render/19183793/max/d/Work/c05.txt",
    "D:\\work/render/19183793\\max/d/Work/c05.txt"
])
def test_convert_path(path):
    """Test get_encode, we can get a expected result."""
    result_path = utils.convert_path(path)
    assert result_path == r"/D/work/render/19183793/max/d/Work/c05.txt"



"""A Python-based API for Using Renderbus cloud rendering houdini service."""

# pylint: disable=import-error
from pkg_resources import DistributionNotFound, get_distribution

from rayvision_c4d.constants import PACKAGE_NAME
from rayvision_log import init_logger

# Initialize the logger.
init_logger(PACKAGE_NAME)

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    # Package is not installed.
    __version__ = '0.0.0-dev.1'



"""Common method for rayvision_c4d API."""

import sys


def get_encode(encode_str, py_version=3):
    """Get the encoding of the string decoding.

    Args:
        encode_str (str, unicode): String.
        py_version (int): Python version, default is 3.

    Returns:
        str: Coding.

    """
    if ((py_version == 2 and isinstance(encode_str, str)) or (
            py_version == 3 and isinstance(encode_str, str))):
        encode = "unicode"
    else:
        for code in ["utf-8", sys.getfilesystemencoding(), "gb18030",
                     "ascii", "gbk", "gb2312"]:
            try:
                encode_str.decode(code, 'ignore')
                return code
            except UnicodeDecodeError:
                pass
        encode = 'utf-8'
    return encode


def str_to_unicode(encode_str, py_version=3):
    """Get the encoding of the string decoding.

    Args:
        encode_str (str, unicode): String.
        py_version (int): Python version, default is 3.

    Returns:
        str: String.

    """
    if (encode_str is None or encode_str == "" or encode_str == 'Null' or
            encode_str == 'null'):
        encode_str = ""
    elif ((py_version == 2 and isinstance(encode_str, str)) or (
            py_version == 3 and isinstance(encode_str, str))):
        pass
    else:
        code = get_encode(encode_str)
        encode_str = encode_str.decode(code, 'ignore')
    return encode_str


def unicode_to_str(str1, logger=None, str_encode='system', py_version=3):
    """Unicode encoded string converted to str."""
    if str1 is None or str1 == "" or str1 == 'Null' or str1 == 'null':
        str1 = ""
    elif ((py_version == 2 and isinstance(str1, str)) or (
            py_version == 3 and isinstance(str1, str))):
        try:
            if str_encode.lower() == 'system':
                str1 = str1.encode(sys.getfilesystemencoding(), 'ignore')
            elif str_encode.lower() == 'utf-8':
                str1 = str1.encode('utf-8', 'ignore')
            elif str_encode.lower() == 'gbk':
                str1 = str1.encode('gbk', 'ignore')
            else:
                str1 = str1.encode(str_encode, 'ignore')
        except UnicodeDecodeError as err_message:
            if logger:
                logger.info('[err]unicode_to_str:encode %s to %s failed',
                            str1, str_encode)
                logger.info(str(err_message))
    elif ((py_version == 2 and isinstance(str1, str)) or (
            py_version == 3 and isinstance(str1, bytes))):
        pass
    else:
        if logger:
            logger.info('%s is not unicode ', str1)
    return str(str1)


def bytes_to_str(str1, logger=None, str_decode='default', py_version=3):
    """Bytes encoded string converted to str."""
    if not ((py_version == 2 and isinstance(str1, str)) or
            (py_version == 3 and isinstance(str1, str))):
        try:
            if str_decode != 'default':
                str1 = str1.decode(str_decode.lower(), 'ignore')
            else:
                try:
                    str1 = str1.decode('utf-8', 'ignore')
                except UnicodeDecodeError:
                    try:
                        str1 = str1.decode('gbk', 'ignore')
                    except UnicodeDecodeError:
                        str1 = str1.decode(sys.getfilesystemencoding(),
                                           'ignore')
        except UnicodeDecodeError as err_message:
            if logger:
                logger.info('[err]bytes_to_str:decode %s to str failed', str1)
                logger.info(str(err_message))
    return str1


def to_gbk(encode_str, py_version):
    """Convert string to gbk code."""
    if ((py_version == 2 and isinstance(encode_str, str
                                        )) or (py_version == 3 and
                                               isinstance(encode_str, str))):
        pass
    else:
        code = get_encode(encode_str)
        encode_str = encode_str.decode(code).encode('GBK', 'ignore')
    return encode_str


def convert_path(path):
    """Convert to the path the server will accept.

    Args:
        path (str): Local file path.
            e.g.:
                "D:/work/render/19183793/max/d/Work/c05/112132P-embery.jpg"

    Returns:
        str: Path to the server.
            e.g.:
                "/D/work/render/19183793/max/d/Work/c05/112132P-embery.jpg"

    """
    lower_path = path.replace('\\', '/')
    if lower_path[1] == ":":
        path_lower = lower_path.replace(":", "")
        path_server = "/" + path_lower
    else:
        path_server = lower_path[1:]

    return path_server



# -*- coding: utf-8 -*-
"""only analyze c4d"""

from rayvision_c4d.analyze_c4d import AnalyzeC4d

analyze_info = {
    "cg_file": r"D:\houdini\cg_file\ybt.c4d",
    "workspace": "c:/workspace",
    "software_version": "R22",
    "project_name": "Project1",
    "plugin_config": {}
}

AnalyzeC4d(**analyze_info).analyse(exe_path=r"C:\Program Files\Maxon Cinema 4D R22\Cinema 4D.exe")